package com.pusong.business.domain.bo;

import com.pusong.business.domain.PsApproverRecord;
import com.pusong.common.core.validate.AddGroup;
import com.pusong.common.core.validate.EditGroup;
import com.pusong.common.mybatis.core.domain.BaseEntity;
import io.github.linpeilie.annotations.AutoMapper;
import lombok.Data;
import lombok.EqualsAndHashCode;
import jakarta.validation.constraints.*;
import java.util.Date;
import com.fasterxml.jackson.annotation.JsonFormat;

/**
 * 审批记录业务对象 ps_approver_record
 *
 * @author wls
 * @date 2024-08-05
 */
@Data
@EqualsAndHashCode
@AutoMapper(target = PsApproverRecord.class)
public class PsApproverRecordBo  {

    /**
     * 主键id
     */
    @NotNull(message = "主键id不能为空", groups = { EditGroup.class })
    private Long id;

    /**
     * 关联业务id
     */
    @NotNull(message = "关联业务id不能为空")
    private String businessId;

    /**
     * 审批业务类型
     */
    @NotBlank(message = "审批业务类型不能为空")
    private String businessType;

    /**
     * 申请日期
     */
    @NotNull(message = "申请日期不能为空")
    private Date applyDate;

    /**
     * 审批日期
     */
    @NotBlank(message = "审批日期不能为空")
    private Date approverDate;

    /**
     * 审批状态（1待审批，2成功，3失败）
     */
    @NotBlank(message = "审批状态（1待审批，2成功，3失败）不能为空")
    private String approverStatus;

    /**
     * 申请说明
     */
    @NotBlank(message = "申请说明不能为空")
    private String applyDesc;

    /**
     * 审批说明
     */
    @NotBlank(message = "审批说明不能为空")
    private String approverDesc;

    /**
     * 修改后数据
     */
    @NotBlank(message = "修改前数据")
    private String updateData;

    /**
     * 客户姓名
     */
    private String customerName;
    /**
     * 客户电话
     */
    private String customMobile;




    /**
     * 合同编码
     */
    private String contractCode;
    /**
     * 回、退款方式
     */
    private String pay;

    /**
     * 申请人
     */
    private String applicant;
    /**
     * 服务签约公司名称
     * 服务公司和签约公司统一使用
     */
    private String companyName;

    /**
     * 销售经理姓名
     * 认领人和销售经理为统一字段
     */
    private String customManagerName;
}
