package com.pusong.business.domain.bo;

import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.pusong.business.domain.PsContractBusiness;
import com.pusong.common.core.validate.AddGroup;
import com.pusong.common.core.validate.EditGroup;
import com.pusong.common.mybatis.core.domain.BaseEntity;
import io.github.linpeilie.annotations.AutoMapper;
import lombok.Data;
import lombok.EqualsAndHashCode;
import jakarta.validation.constraints.*;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 合同服务类别业务对象 ps_contract_business
 *
 * @author wls
 * @date 2024-07-29
 */
@Data
@EqualsAndHashCode
@AutoMapper(target = PsContractBusiness.class)
public class PsContractBusinessBo  {

    /**
     * 主键id
     */
    @TableId(value = "id")
    private Long id;
    /**
     * 合同编号
     */
    @NotNull(message = "合同编号不能为空")
    private String contractCode;

    /**
     * 服务类型
     */
    @NotBlank(message = "服务类型不能为空", groups = { AddGroup.class, EditGroup.class })
    private String businessType;
    /**
     * 服务类型名称，生成pdf使用
     */
    private String businessTypeName;

    /**
     * 服务总金额
     */
    @NotNull(message = "服务总金额不能为空", groups = { AddGroup.class, EditGroup.class })
    private BigDecimal businessAmount;
    /**
     * 服务类型备注
     */
    private String businessDesc;
    /**
     * 结束时间
     */
    @JsonFormat(pattern ="yyyy-MM-dd")
    private Date endDate;
    /**
     * 开始时间
     */
    @JsonFormat(pattern ="yyyy-MM-dd")
    private Date startDate;
    /**
     * 服务公司id
     */
    private Long companyId;
    /**
     * 合同服务项目
     */
    private List<PsContractBusinessDetailBo> detailBos;

    /**
     * 是否代账
     */
    private String isProxy;
    /**
     * 是否老账新接
     */
    private String isOld;

}
