package com.pusong.business.domain.bo;

import com.pusong.business.domain.PsContractPay;
import com.pusong.common.core.validate.AddGroup;
import com.pusong.common.core.validate.EditGroup;
import com.pusong.common.mybatis.core.domain.BaseEntity;
import io.github.linpeilie.annotations.AutoMapper;
import lombok.Data;
import lombok.EqualsAndHashCode;
import jakarta.validation.constraints.*;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Date;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.springframework.format.annotation.DateTimeFormat;

/**
 * 合同回款记录业务对象 ps_contract_pay
 *
 * @author wls
 * @date 2024-08-01
 */
@Data
@EqualsAndHashCode
@AutoMapper(target = PsContractPay.class)
public class PsContractPayBo  {

    /**
     * 主键id
     */
    @NotNull(message = "主键id不能为空", groups = { EditGroup.class })
    private Long id;

    /**
     * 合同编码
     */
    @NotNull(message = "合同编码", groups = { AddGroup.class, EditGroup.class })
    private String contractCode;
    /**
     * 业务类型1回款2退款
     */
    @NotNull(message = "业务类型1回款2退款", groups = { AddGroup.class, EditGroup.class })
    private String businessType;
    /**
     * 回款金额
     */
    @NotNull(message = "回款金额不能为空", groups = { AddGroup.class, EditGroup.class })
    private BigDecimal money;

    /**
     * 回款日期
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date payDate;


    /**
     * 回款类型（1全款，2分期）
     */
    private String payType;

    /**
     * 支付方式（1扫码，2对公，3现金，4对私）
     */
    @NotBlank(message = "支付方式（1扫码，2对公，3现金，4对私）不能为空", groups = { AddGroup.class, EditGroup.class })
    private String payMode;

    /**
     * 收款主体（退款时对公账号）
     */
    private String gathMain;

//    /**
//     * 回款凭证(多个时用逗号分隔)
//     */
//    @NotBlank(message = "回款凭证(多个时用逗号分隔)不能为空", groups = { AddGroup.class, EditGroup.class })
//    private String payCertification;

    /**
     * 回款/退款备注
     */
    private String payDesc;

    /**
     * 修改回款时的备注  ：回款备注
     */
    private String updateDesc;


}
