package com.pusong.business.domain.vo;

import java.math.BigDecimal;
import java.util.Date;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.pusong.business.domain.PsApproverRecord;
import com.alibaba.excel.annotation.ExcelIgnoreUnannotated;
import com.alibaba.excel.annotation.ExcelProperty;
import com.pusong.common.excel.annotation.ExcelDictFormat;
import com.pusong.common.excel.convert.ExcelDictConvert;
import com.pusong.common.translation.annotation.Translation;
import com.pusong.common.translation.constant.TransConstant;
import io.github.linpeilie.annotations.AutoMapper;
import jakarta.validation.constraints.NotBlank;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;
import java.util.Date;
import java.util.List;


/**
 * 审批记录视图对象 ps_approver_record
 *
 * @author wls
 * @date 2024-08-05
 */
@Data
@ExcelIgnoreUnannotated
@AutoMapper(target = PsApproverRecord.class)
public class PsApproverRecordVo implements Serializable {

    @Serial
    private static final long serialVersionUID = 1L;

    /**
     * 客户id
     */
    private Long customId;
    /**
     * 主键id
     */
    private Long id;
    /**
     * 合同编码
     */
    private String contractCode;
    /**
     * 用户id
     */
    private String cusId;
    /**
     * 公司id
     */
    private String comId;
    /**
     * 审批业务类型
     */
    private String businessType;
    /**
     * 关联业务id
     */
    private String businessId;
    /**
     * 申请日期
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date applyDate;

    /**
     * 审批日期
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date approverDate;

    /**
     * 审批状态（1待审批，2成功，3失败）
     */
    private String approverStatus;

    /**
     * 申请说明
     */
    private String applyDesc;

    /**
     * 审批说明
     */
    private String approverDesc;

    /**
     * 修改后数据
     */
    private String updateData;
    /**
     * 创建者
     */
    @Translation(type  = TransConstant.USER_ID_TO_NICKNAME)
    private Long createBy;
    /**
     * 更新者
     */
    @Translation(type  = TransConstant.USER_ID_TO_NICKNAME)
    private Long updateBy;

    /**
     * 更新时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updateTime;

    /**
     * 合同文件id
     */
    private String pdfId;
    /**
     * 合同所属销售经理
     */
    @Translation(type  = TransConstant.USER_ID_TO_NICKNAME)
    private Long customManager;

    /**
     * 签约公司id
     */
    private Long companyId;
    /**
     * 公司名
     */
    private String companyName;

    /**
     * 签约个人时联系人姓名
     */
    private String contactPersonName;

    /**
     * 签约个人时联系人手机号
     */
    private String contactPersonPhone;

    /**
     * 签约个人时联系人身份证号
     */
    private String contactPersonIdcard;
    /**
     * 签约公司信息（不会有服务类别）
     */
    private PsCompanyInfoVo psCompanyInfoVo;
//
//    /**
//     * 公司所属区域
//     */
//    private String companyAdress;

    /**
     * 服务公司信息
     */
    private List<PsCompanyInfoVo> servicePsCompanyInfoVo;


    /**
     * 客户名
     */
    private String customName;
    /**
     * 客户电话
     */
    private String customMobile;

    /**
     * 回传合同文件id
     */
    private String rollBackPdf;
    /**
     * 服务类别名（多个是用逗号分隔）
     */
    @Translation(type = TransConstant.DICT_TYPE_TO_LABEL,other = "contract_type")
    private String businessTypeName;
    /**
     * 合同总金额
     */
    private BigDecimal contractAmount;

    @Translation(type = TransConstant.DICT_TYPE_TO_LABEL,other = "contract_status", mapper="contractStatus")
    private String contractStatusName;

    private String contractStatus;

    /**
     * 已付金额
     */
    private BigDecimal payMoney;

    /**
     * 未付金额
     */
    private BigDecimal residualMoney;
    /**
     * 回款周期（天）
     */
    private Integer period;
    /**
     * 介绍人
     */
    @Translation(type  = TransConstant.USER_ID_TO_NICKNAME)
    private Long customIntroducer;
    /**
     * 客户来源
     */
    private String customSource;

    /**
     * 公司集合（公海审批使用）
     */
    private List<PsCompanyInfoVo> companyInfoList;
    /**
     * 公司数量
     */
    private Integer companyNum;

    /**
     * 支付凭证
     */
    private String payCertification;

    /**
     * 退款详细信息
     */
    private PsContractPayVo psContractPayVo;

}
