package com.pusong.business.domain.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.pusong.business.domain.PsCustomInfo;
import com.alibaba.excel.annotation.ExcelIgnoreUnannotated;
import com.alibaba.excel.annotation.ExcelProperty;
import com.pusong.common.excel.annotation.ExcelDictFormat;
import com.pusong.common.excel.convert.ExcelDictConvert;
import com.pusong.common.translation.annotation.Translation;
import com.pusong.common.translation.constant.TransConstant;
import io.github.linpeilie.annotations.AutoMapper;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;


/**
 * 客户基本信息视图对象 ps_custom_info
 *
 * @author Lion Li
 * @date 2024-07-25
 */
@Data
@ExcelIgnoreUnannotated
@AutoMapper(target = PsCustomInfo.class)
public class PsCustomInfoVo implements Serializable {

    @Serial
    private static final long serialVersionUID = 1L;

    /**
     * 
     */
    private Long id;

    /**
     * 客户名称
     */
    private String customName;

    /**
     * 客户电话客户电话(多个用英文逗号分割)
     */
    private String customMobile;

    /**
     * 客户来源
     */
    @ExcelProperty(value = "客户来源", converter = ExcelDictConvert.class)
    @ExcelDictFormat(dictType = "custom_source")
    private String customSource;

    /**
     * 客户联系地址
     */
    private String customAdressDetail;

    /**
     * 所属客户经理
     */
    private String customManager;
    /**
     * 所属客户经理
     */
    private String customManagerName;

    /**
     * 客户级别
     */
    @ExcelProperty(value = "客户级别", converter = ExcelDictConvert.class)
    @ExcelDictFormat(dictType = "custom_level")
    private String customLevel;

    /**
     * 客户状态
     */
    @ExcelProperty(value = "客户状态", converter = ExcelDictConvert.class)
    @ExcelDictFormat(dictType = "customer_status")
    private String customStatus;

    /**
     * 介绍人-仅支持本司内客户
     */
    private Long customIntroducer;

    /**
     * 介绍人姓名-仅支持本司内客户
     */
    private String customIntroducerName;

    /**
     * 拉黑标识
     */
    private String black;
    /**
     * 拉黑备注
     */
    private String blackDesc;
    /**
     * 颜色
     */
    private String color;

    /**
     * 创建时间
     */
    @ExcelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    private Date createTime;

    /**
     * 修改时间
     */
    @ExcelProperty(value = "修改时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    private Date updateTime;

    /**
     * 总报价金额
     */
    private BigDecimal price;

    /**
     * 最新的回访记录
     */
    private String callBack;

    /**
     * 最早的公司名称
     */
    private String companyName;

    /**
     * 公司数量
     */
    private Integer companyNum;
    /**
     * 历史的所属客户经理
     */
    private String historyCustomManager;
    /**
     * 历史的所属客户经理
     */
    @Translation(type = TransConstant.USER_ID_TO_NICKNAME,mapper = "historyCustomManager")
    private String historyCustomManagerName;
    /**
     * 是否已生成合同（1是0否）
     */
    private Integer haveContract;
    /**
     * 合同编码（最新的）
     */
    private String contractCode;
    /**
     * 派单日期
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    private Date startDate;
    /**
     * 任务进行天数
     */
    private int numOfDat;

    /**
     * 公海认领审批失败描述
     */
    private String publicDesc;
    /**
     * 报价金额DTO
     */
    private List<PsCustomPriceVo> psCustomPriceVo;

    /**
     * 回访记录DTO
     */
    private List<PsCustomCallbackVo> psCustomCallbackVo;

    /**
     * 公司信息DTO
     */
    private List<PsCompanyInfoVo> psCompanyInfoVos;
}
