package com.pusong.business.domain.vo;

import com.alibaba.excel.annotation.ExcelIgnoreUnannotated;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.pusong.business.domain.PsTaskMain;
import com.pusong.common.translation.annotation.Translation;
import com.pusong.common.translation.constant.TransConstant;
import io.github.linpeilie.annotations.AutoMapper;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;


/**
 * 客户档案列表返回VO
 *
 * @author wls
 * @date 2024-08-07
 */
@Data
@ExcelIgnoreUnannotated
public class PsCustomerRecordVo implements Serializable {

    @Serial
    private static final long serialVersionUID = 1L;

    /**
     * 客户表主键id
     */
    private Long id;
    /**
     * 合同编码
     */
    private String contractCode;
    /**
     * 客户名称
     */
    private String customName;

    /**
     * 客户电话客户电话(多个用英文逗号分割)
     */
    private String customMobile;
    /**
     * 服务类别名（多个是用逗号分隔）
     */
    @Translation(type = TransConstant.DICT_TYPE_TO_LABEL,other = "contract_type")
    private String businessTypeName;

    /**
     * 派单时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    private Date startDate;
    /**
     * 任务进行天数
     */
    private int numOfDat;
    /**
     * 账套号
     */
    private String financeAccountNo;
    /**
     * 销售经理
     */
    @Translation(type = TransConstant.USER_ID_TO_NICKNAME)
    private String customManager;
    /**
     * 所属区域
     */
    private String companyAdress;
    /**
     * 公司名称
     */
    private String companyName;
    /**
     * 纳税人识别号
     */
    private String customerNum;
    /**
     * 纳税人资格
     */
    private String customerCert;
    /**
     * 法人姓名
     */
    private String legalPersonName;

    /**
     * 法人证件号
     */
    private String legalPersonIdcard;

    /**
     * 法人电话号
     */
    private String legalPersonPhone;
    /**
     * 当前服务人
     */
    private String executors;
    /**
     * 当前服务人名字
     */
    @Translation(type = TransConstant.USER_ID_TO_NICKNAME,mapper = "executors")
    private String executorName;

    /**
     * 签约备注
     */
    private String signDesc;

    /**
     * 合同完成日期
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    private Date finishDate;

    /**
     * 审批状态
     */
    private String approverStatus;

    /**
     * 客户创建时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    private Date createTime;
    /**
     * 合同总金额
     */
    private BigDecimal contractAmount;
    /**
     * 确认单
     */
    private String confirmOssid;

    /**
     * 客户状态
     */
    private String customStatus;
    /**
     * 渠道
     */
    private String customSource;
    /**
     * 介绍人
     */
    private String customIntroducer;
    /**
     * 免费工单审批失败描述
     */
    private String freeFailDesc;

    /**
     * 回访记录vo（已完成客户列表使用）
     */
    private List<PsCustomCallbackVo> psCustomCallbackVo;
}
