package com.pusong.business.domain.vo;

import java.util.Date;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.pusong.business.domain.PsTaskAppoint;
import com.pusong.business.domain.PsTaskMain;
import com.alibaba.excel.annotation.ExcelIgnoreUnannotated;
import com.pusong.common.excel.annotation.ExcelDictFormat;
import com.pusong.common.excel.convert.ExcelDictConvert;
import com.pusong.common.translation.annotation.Translation;
import com.pusong.common.translation.constant.TransConstant;
import io.github.linpeilie.annotations.AutoMapper;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;
import java.util.Date;
import java.util.List;


/**
 * 主任务视图对象 ps_task_main
 *
 * @author wls
 * @date 2024-08-07
 */
@Data
@ExcelIgnoreUnannotated
@AutoMapper(target = PsTaskMain.class)
public class PsTaskMainVo implements Serializable {

    @Serial
    private static final long serialVersionUID = 1L;

    /**
     * 主键id
     */
    private Long id;
    /**
     * 主任务主键id
     */
    private Long mainId;
    /**
     * 合同编码
     */
    private String contractCode;
    /**
     * 任务类型（1合同任务,2免费工单,3发票）
     */
    private String taskType;

    /**
     * 派单时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    private Date startDate;
    /**
     * 任务进行天数
     */
    private int numOfDat;
    /**
     * 免费工单的服务类别
     */
    private String businessType;

    /**
     * 免费工单的服务内容
     */
    private String businessProject;

    /**
     * 主任务状态
     */
    private String taskStatus;
    /**
     * 主任务状态
     */
    private String taskStatusDesc;
    /**
     * 子任务状态
     */
    private String appointStatus;
    /**
     * 服务类别名（多个是用逗号分隔）
     */
    @Translation(type = TransConstant.DICT_TYPE_TO_LABEL,other = "contract_type")
    private String businessTypeName;
    /**
     * 合同信息
     */
    private TaskContractVo contractVo;
    /**
     * 公司信息
     */
    private PsCompanyInfoVo companyInfoVo;
    /**
     * 客户信息
     */
    private PsCustomInfoVo customInfoVo;

    /**
     * 执行者
     */
    @Translation(type = TransConstant.USER_ID_TO_NICKNAME)
    private String executors;

    /**
     * 支付状态描述（全款、半款）
     */
    private String payStautsDesc;
    /**
     * 任务资料信息
     */
    private List<PsTaskMediaVo> taskMediaVoList;

    /**
     * 发票
     */
    private String invoice;

    /**
     * 外勤是否派单1是0否
     */
    private String outAppo;
    /**
     * 内勤是否派单1是0否
     */
    private String inAppo;
    /**
     * 特勤是否派单1是0否
     */
    private String secAppo;
    /**
     * 任务备注,子任务里的备注，queryChildTaskPageList查询的时候存储到这个字段了
     */
    private String taskDesc;

    /**
     * 任务备注，存储了作废原因
     */
    private String comment;

    /**
     * 任务备注,子任务里的工作内容描述，queryChildTaskPageList查询的时候存储到这个字段了
     */
    private String taskContentDesc;

    /**
     * 扩展: 类型未开发票时 发票内容
     */
    private String extend;

    /**
     * 发票信息
     */
    private PsInvoiceVo psInvoiceVo;

    /**
     * 服务公司信息
     */
    private PsCompanyInfoVo psCompanySerivceVo;

    /**
     * 服务公司id
     */
    private Long serviceCompanyId;

    private List<PsTaskAppointVo> psTaskAppointVoList;
}

