package com.pusong.business.service;

import com.pusong.business.domain.bo.PsCompanyQueryBo;
import com.pusong.business.domain.vo.PsCompanyInfoVo;
import com.pusong.business.domain.bo.PsCompanyInfoBo;
import com.pusong.common.mybatis.core.page.TableDataInfo;
import com.pusong.common.mybatis.core.page.PageQuery;

import java.util.Collection;
import java.util.List;

/**
 * 公司信息Service接口
 *
 * @author wls
 * @date 2024-07-29
 */
public interface IPsCompanyInfoService {


    /**
     * 查询公司信息
     *
     * @param id 主键
     * @return 合同基本信息
     */
    PsCompanyInfoVo queryById(Long id);

    PsCompanyInfoVo queryVoAndProxyById(Long id, String contractCode);

    List<PsCompanyInfoVo> queryByIds(List<Long> id);

    List<PsCompanyInfoVo> queryVoAndProxyByIds(List<Long> ids, String contractCode);

    /**
     * 查询符合条件的公司信息列表
     *
     * @param bo 查询条件
     * @return 公司信息列表
     */
    List<PsCompanyInfoVo> queryList(PsCompanyInfoBo bo);

    /**
     * 查询符合条件的公司信息列表
     *
     * @param bo 查询条件
     * @return 公司信息列表
     */
    TableDataInfo<PsCompanyInfoVo> queryPageList(PsCompanyQueryBo bo, PageQuery pageQuery);

    TableDataInfo<PsCompanyInfoVo> queryServiceCompany(PsCompanyQueryBo bo, PageQuery pageQuery);

    TableDataInfo<PsCompanyInfoVo> searchServiceCompany(String keyWorld);

    /**
     * 新增公司信息
     *
     * @param bo 公司信息
     * @return 是否新增成功
     */
    Boolean saveBo(PsCompanyInfoBo bo);

    /**
     * 保存合同公司关联表
     * @param list
     * @param contractCode
     */
    void saveContractCompany(List<PsCompanyInfoBo> list,String contractCode );
    /**
     * 批量新增公司信息
     *
     * @param bos 公司信息
     * @return 是否新增成功
     */
    Boolean saveBatchBo(List<PsCompanyInfoBo> bos);


    /**
     * 校验并批量删除公司信息信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);

    /**
     * 根据合同编码查询服务公司信息
     */
    List<PsCompanyInfoVo> getServiceCompany(String controctCode);

    void terminationService(Long serviceCompanyId);
}
