package com.pusong.business.service;

import com.pusong.business.domain.PsContractPay;
import com.pusong.business.domain.vo.PsContractPayVo;
import com.pusong.business.domain.bo.PsContractPayBo;
import com.pusong.business.domain.vo.PsRefundVo;
import com.pusong.business.enums.PayStatusEnum;
import com.pusong.common.core.utils.MapstructUtils;
import com.pusong.common.mybatis.core.page.TableDataInfo;
import com.pusong.common.mybatis.core.page.PageQuery;
import jakarta.validation.constraints.NotNull;
import org.springframework.web.multipart.MultipartFile;

import java.util.Collection;
import java.util.List;

/**
 * 合同回款记录Service接口
 *
 * @author wls
 * @date 2024-08-01
 */
public interface IPsContractPayService {

    /**
     * 查询合同回款记录列表
     *
     * @param contractCode        合同编码
     * @return 合同回款记录分页列表
     */
    List<PsContractPayVo> queryListByContractCode(String contractCode, String business, PayStatusEnum enu);


    /**
     * 查询合同所有支付记录列表
     *
     * @param contractCode        合同编码
     * @return
     */
    List<PsContractPayVo> queryAllListByContractCode(String contractCode);

    /**
     * 添加支付记录
     * @param files 支付凭证
     * @param payBo 支付信息
     */
    void addPay(List<MultipartFile> files, PsContractPayBo payBo);
    /**
     * 添加退款记录
     * @param files 支付凭证
     * @param payBo 支付信息
     */
    void returnPay(List<MultipartFile> files, PsContractPayBo payBo);
    /**
     * 根据主键id查询回款记录
     *
     * @param id 查询条件
     * @return 合同回款记录列表
     */
    PsContractPayVo queryById(Long id);

    /**
     * 根据id修改合同回款记录
     *
     * @param pay 合同回款记录
     * @return 是否修改成功
     */
    Boolean updateById(PsContractPay pay);




    /**
     * 退款列表查询
     *
     * @return 合同回款记录列表
     */
    TableDataInfo<PsRefundVo> queryRefundList(String customManageName,String companyName,String mobile,String name, PageQuery pageQuery,String payStatus);

    /**
     * 完成退款
     * @param files 附件图片
     * @param id 任务id
     * @return
     */
    String refundFinish(List<MultipartFile> files, Long id);

    /*  *//**
     * 查询合同回款记录
     *
     * @param id 主键
     * @return 合同回款记录
     *//*
    PsContractPayVo queryById(Long id);

    *//**
     * 分页查询合同回款记录列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 合同回款记录分页列表
     *//*
    TableDataInfo<PsContractPayVo> queryPageList(PsContractPayBo bo, PageQuery pageQuery);

    *//**
     * 查询符合条件的合同回款记录列表
     *
     * @param bo 查询条件
     * @return 合同回款记录列表
     *//*
    List<PsContractPayVo> queryList(PsContractPayBo bo);

    *//**
     * 新增合同回款记录
     *
     * @param bo 合同回款记录
     * @return 是否新增成功
     *//*
    Boolean insertByBo(PsContractPayBo bo);

    *//**
     * 修改合同回款记录
     *
     * @param bo 合同回款记录
     * @return 是否修改成功
     *//*
    Boolean updateByBo(PsContractPayBo bo);

    *//**
     * 校验并批量删除合同回款记录信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     *//*
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);*/
}
