package com.pusong.business.service.impl;

import com.pusong.business.domain.PsSalaryContract;
import com.pusong.business.domain.vo.PsSalaryContractVo;
import com.pusong.business.mapper.PsSalaryContractMapper;
import com.pusong.common.core.utils.MapstructUtils;
import com.pusong.common.mybatis.core.page.TableDataInfo;
import com.pusong.common.mybatis.core.page.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import com.pusong.business.domain.bo.PsSalaryBo;
import com.pusong.business.domain.vo.PsSalaryVo;
import com.pusong.business.domain.PsSalary;
import com.pusong.business.mapper.PsSalaryMapper;
import com.pusong.business.service.IPsSalaryService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 员工提成提成Service业务层处理
 *
 * @author wls
 * @date 2024-08-23
 */
@RequiredArgsConstructor
@Service
public class PsSalaryServiceImpl implements IPsSalaryService {

    private final PsSalaryMapper baseMapper;
    private final PsSalaryContractMapper salaryContractMapper;

    /**
     * 分页查询员工提成提成列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 员工提成提成分页列表
     */
    @Override
    public TableDataInfo<PsSalaryVo> queryPageList(PsSalaryBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<PsSalary> lqw = buildQueryWrapper(bo);
        Page<PsSalaryVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }
    private LambdaQueryWrapper<PsSalary> buildQueryWrapper(PsSalaryBo bo) {
        LambdaQueryWrapper<PsSalary> lqw = Wrappers.lambdaQuery();
        lqw.le(bo.getEndDate() != null,PsSalary::getSalaryDate,bo.getEndDate());
        lqw.ge(bo.getStartDate() != null,PsSalary::getSalaryDate,bo.getStartDate());
        lqw.exists(StringUtils.isNotBlank(bo.getName()),"select 1 from sys_user su where su.user_id = ps_salary.user_id and su.nick_name like %"+bo.getName()+"%");
        return lqw;
    }
    /**
     * 提成关联合同查询
     * @param id
     * @param status
     * @param pageQuery
     * @return
     */
    public TableDataInfo<PsSalaryContractVo> queryContractPageList(Long id, String status, PageQuery pageQuery){
        Page<PsSalaryContractVo> result = salaryContractMapper.selectVoPage( pageQuery.build(),Wrappers.<PsSalaryContract>lambdaQuery()
                .eq(PsSalaryContract::getSalaryId ,id)
                .eq(StringUtils.isNotBlank(status),PsSalaryContract::getStatus,status));
        return TableDataInfo.build(result);
    }


//    /**
//     * 查询员工提成提成
//     *
//     * @param id 主键
//     * @return 员工提成提成
//     */
//    @Override
//    public PsSalaryVo queryById(Long id){
//        return baseMapper.selectVoById(id);
//    }
//
//    /**
//     * 查询符合条件的员工提成提成列表
//     *
//     * @param bo 查询条件
//     * @return 员工提成提成列表
//     */
//    @Override
//    public List<PsSalaryVo> queryList(PsSalaryBo bo) {
//        LambdaQueryWrapper<PsSalary> lqw = buildQueryWrapper(bo);
//        return baseMapper.selectVoList(lqw);
//    }
//
//    private LambdaQueryWrapper<PsSalary> buildQueryWrapper(PsSalaryBo bo) {
//        Map<String, Object> params = bo.getParams();
//        LambdaQueryWrapper<PsSalary> lqw = Wrappers.lambdaQuery();
//        lqw.eq(bo.getUserId() != null, PsSalary::getUserId, bo.getUserId());
//        lqw.eq(bo.getServiceProject() != null, PsSalary::getServiceProject, bo.getServiceProject());
//        lqw.eq(StringUtils.isNotBlank(bo.getSalaryRatio()), PsSalary::getSalaryRatio, bo.getSalaryRatio());
//        return lqw;
//    }
//
//    /**
//     * 新增员工提成提成
//     *
//     * @param bo 员工提成提成
//     * @return 是否新增成功
//     */
//    @Override
//    public Boolean insertByBo(PsSalaryBo bo) {
//        PsSalary add = MapstructUtils.convert(bo, PsSalary.class);
//        validEntityBeforeSave(add);
//        boolean flag = baseMapper.insert(add) > 0;
//        if (flag) {
//            bo.setId(add.getId());
//        }
//        return flag;
//    }
//
//    /**
//     * 修改员工提成提成
//     *
//     * @param bo 员工提成提成
//     * @return 是否修改成功
//     */
//    @Override
//    public Boolean updateByBo(PsSalaryBo bo) {
//        PsSalary update = MapstructUtils.convert(bo, PsSalary.class);
//        validEntityBeforeSave(update);
//        return baseMapper.updateById(update) > 0;
//    }
//
//    /**
//     * 保存前的数据校验
//     */
//    private void validEntityBeforeSave(PsSalary entity){
//        //TODO 做一些数据校验,如唯一约束
//    }
//
//    /**
//     * 校验并批量删除员工提成提成信息
//     *
//     * @param ids     待删除的主键集合
//     * @param isValid 是否进行有效性校验
//     * @return 是否删除成功
//     */
//    @Override
//    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
//        if(isValid){
//            //TODO 做一些业务上的校验,判断是否需要校验
//        }
//        return baseMapper.deleteBatchIds(ids) > 0;
//    }
}
