package com.pusong.business.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.pusong.common.core.utils.MapstructUtils;
import com.pusong.common.core.utils.StringUtils;
import com.pusong.common.mybatis.core.page.TableDataInfo;
import com.pusong.common.mybatis.core.page.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import com.pusong.business.domain.bo.PsTaskMediaBo;
import com.pusong.business.domain.vo.PsTaskMediaVo;
import com.pusong.business.domain.PsTaskMedia;
import com.pusong.business.mapper.PsTaskMediaMapper;
import com.pusong.business.service.IPsTaskMediaService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 任务资料Service业务层处理
 *
 * @author wls
 * @date 2024-08-07
 */
@RequiredArgsConstructor
@Service
public class PsTaskMediaServiceImpl implements IPsTaskMediaService {

    private final PsTaskMediaMapper baseMapper;

    /**
     * 查询任务资料
     *
     * @param customerId 客户id
     * @param companyId 公司id
     * @return 任务资料
     */
    @Override
    public List<PsTaskMediaVo> queryMediaList(Long customerId,Long companyId){
        PsTaskMediaBo bo = new PsTaskMediaBo();
        bo.setCustomId(customerId);
        bo.setCompanyId(companyId);
        LambdaQueryWrapper<PsTaskMedia> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }
    private LambdaQueryWrapper<PsTaskMedia> buildQueryWrapper(PsTaskMediaBo bo) {
        LambdaQueryWrapper<PsTaskMedia> lqw = Wrappers.lambdaQuery();
        lqw.eq(bo.getCustomId() != null, PsTaskMedia::getCustomId, bo.getCustomId());
        lqw.eq(bo.getCompanyId() != null, PsTaskMedia::getCompanyId, bo.getCompanyId());
        lqw.eq(StringUtils.isNotBlank(bo.getMediaType()), PsTaskMedia::getMediaType, bo.getMediaType());
        lqw.eq(StringUtils.isNotBlank(bo.getMediaOssId()), PsTaskMedia::getMediaOssId, bo.getMediaOssId());
        return lqw;
    }

    public void saveMediaList(List<PsTaskMediaBo> voList){
        if(CollectionUtils.isEmpty(voList)){
            return;
        }
        List<PsTaskMedia> save = MapstructUtils.convert(voList, PsTaskMedia.class);
        baseMapper.delete(Wrappers.<PsTaskMedia>lambdaQuery().eq(PsTaskMedia::getCustomId, save.get(0).getCustomId())
                .eq(PsTaskMedia::getCompanyId, save.get(0).getCompanyId()));
        baseMapper.insertOrUpdateBatch(save);
    }
//    /**
//     * 分页查询任务资料列表
//     *
//     * @param bo        查询条件
//     * @param pageQuery 分页参数
//     * @return 任务资料分页列表
//     */
//    @Override
//    public TableDataInfo<PsTaskMediaVo> queryPageList(PsTaskMediaBo bo, PageQuery pageQuery) {
//        LambdaQueryWrapper<PsTaskMedia> lqw = buildQueryWrapper(bo);
//        Page<PsTaskMediaVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
//        return TableDataInfo.build(result);
//    }
//
//    /**
//     * 查询符合条件的任务资料列表
//     *
//     * @param bo 查询条件
//     * @return 任务资料列表
//     */
//    @Override
//    public List<PsTaskMediaVo> queryList(PsTaskMediaBo bo) {
//        LambdaQueryWrapper<PsTaskMedia> lqw = buildQueryWrapper(bo);
//        return baseMapper.selectVoList(lqw);
//    }
//
//
//
//    /**
//     * 新增任务资料
//     *
//     * @param bo 任务资料
//     * @return 是否新增成功
//     */
//    @Override
//    public Boolean insertByBo(PsTaskMediaBo bo) {
//        PsTaskMedia add = MapstructUtils.convert(bo, PsTaskMedia.class);
//        validEntityBeforeSave(add);
//        boolean flag = baseMapper.insert(add) > 0;
//        if (flag) {
//            bo.setId(add.getId());
//        }
//        return flag;
//    }
//
//    /**
//     * 修改任务资料
//     *
//     * @param bo 任务资料
//     * @return 是否修改成功
//     */
//    @Override
//    public Boolean updateByBo(PsTaskMediaBo bo) {
//        PsTaskMedia update = MapstructUtils.convert(bo, PsTaskMedia.class);
//        validEntityBeforeSave(update);
//        return baseMapper.updateById(update) > 0;
//    }
//
//    /**
//     * 保存前的数据校验
//     */
//    private void validEntityBeforeSave(PsTaskMedia entity){
//        //TODO 做一些数据校验,如唯一约束
//    }
//
//    /**
//     * 校验并批量删除任务资料信息
//     *
//     * @param ids     待删除的主键集合
//     * @param isValid 是否进行有效性校验
//     * @return 是否删除成功
//     */
//    @Override
//    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
//        if(isValid){
//            //TODO 做一些业务上的校验,判断是否需要校验
//        }
//        return baseMapper.deleteBatchIds(ids) > 0;
//    }
}
